@@ -74,6 +74,7 @@ extern void free_ms_biff_bof_data (BIFF_BOF_DATA *data) ;
 #define BIFF_FOOTER                     0x15
 #define BIFF_EXTERNCOUNT                0x16 /* number of external references*/
 #define BIFF_EXTERNSHEET                0x17
+#define BIFF_FORMAT                     0x1e
 #define BIFF_ARRAY                      0x21
 #define BIFF_FONT                       0x31
 #define BIFF_XF_OLD                     0x43
@@ -81,8 +82,10 @@ extern void free_ms_biff_bof_data (BIFF_BOF_DATA *data) ;
 #define BIFF_RK                         0x7e
 #define BIFF_BOUNDSHEET                 0x85
 #define BIFF_PALETTE                    0x92
+#define BIFF_MULRK                      0xbd
 #define BIFF_MULBLANK                   0xbe
 #define BIFF_RSTRING                    0xd6
+#define BIFF_DBCELL                     0xd7
 #define BIFF_XF                         0xe0
 
 #define BIFF_SST                        0xfc
