@@ -28,12 +28,14 @@ void ms_excel_fixup_array_formulae (MS_EXCEL_SHEET *sheet) ;
 #define FORMULA_PTG_MISSARG            0x16
 #define FORMULA_PTG_STR                0x17
 #define FORMULA_PTG_ATTR               0x19
+#define FORMULA_PTG_ERR                0x1c
 #define FORMULA_PTG_BOOL               0x1d
 #define FORMULA_PTG_INT                0x1e
 #define FORMULA_PTG_NUM                0x1f /* 8 byte IEEE floating point number */
 
 #define FORMULA_PTG_FUNC               0x21
 #define FORMULA_PTG_FUNC_VAR           0x22
+#define FORMULA_PTG_NAME               0x23
 #define FORMULA_PTG_REF                0x24
 #define FORMULA_PTG_AREA               0x25
 #define FORMULA_PTG_MEM_AREA           0x26
