@@ -713,7 +713,13 @@ cell_copy (Cell *cell)
 	string_ref      (new_cell->text);
 	
 	new_cell->style = style_duplicate (new_cell->style);
-	new_cell->value = value_duplicate (new_cell->value);
+
+	/*
+	 * The cell->value can be NULL if the cell contains
+	 * an error
+	 */
+	if (new_cell->value)
+		new_cell->value = value_duplicate (new_cell->value);
 
 	new_cell->comment = NULL;
 	if (cell->comment)
@@ -852,6 +858,28 @@ cell_set_format (Cell *cell, char *format)
 	cell_queue_redraw (cell);
 }
 
+void
+cell_set_format_from_style (Cell *cell, StyleFormat *style_format)
+{
+	g_return_if_fail (cell != NULL) ;
+	g_return_if_fail (cell->value) ;
+	g_return_if_fail (style_format != NULL) ;
+	
+	cell_modified (cell);
+	cell_queue_redraw (cell);
+	
+	/* Change the format */
+	style_format_unref (cell->style->format);
+	style_format_ref (style_format) ;
+
+	cell->style->format = style_format ;
+	cell->flags |= CELL_FORMAT_SET;
+	
+	/* re-render the cell text */
+	cell_render_value (cell);
+	cell_queue_redraw (cell);
+}
+
 void
 cell_comment_reposition (Cell *cell)
 {
