@@ -1,5 +1,5 @@
-#ifndef ITEM_CURSOR_H
-#define ITEM_CURSOR_H
+#ifndef GNUMERIC_ITEM_CURSOR_H
+#define GNUMERIC_ITEM_CURSOR_H
 
 #define ITEM_CURSOR(obj)          (GTK_CHECK_CAST((obj), item_cursor_get_type (), ItemCursor))
 #define ITEM_CURSOR_CLASS(k)      (GTK_CHECK_CLASS_CAST ((k), item_cursor_get_type ()))
@@ -27,10 +27,14 @@ typedef struct {
 	int      tag;
 
 	/*
-	 * for the autofill mode:
-	 * Where the action started (base_x, base_y) and the
-	 * width and heigth of the selection when the autofill
-	 * started
+	 * For the autofill mode:
+	 *     Where the action started (base_x, base_y) and the
+	 *     width and heigth of the selection when the autofill
+	 *     started.
+	 *
+	 * For the anted mode:
+	 *     base_col and base_row are used to keep track of where
+	 *     the selection was started.
 	 */
 	int      base_x, base_y;
 	int      base_col, base_row;
@@ -57,5 +61,4 @@ void item_cursor_set_bounds     (ItemCursor *item_cursor,
 void item_cursor_set_visibility (ItemCursor *item_cursor,
 				 int visible);
 
-#endif
-
+#endif /* GNUMERIC_ITEM_CURSOR_H */
