@@ -67,6 +67,8 @@ typedef struct {
 	 * to the asyncronous paste callback
 	 */
 	void       *clipboard_paste_callback_data;
+
+	void       *toolbar;
 } Workbook;
 
 typedef struct {
@@ -220,6 +222,8 @@ void        sheet_cell_comment_unlink     (Cell *cell);
 
 void        sheet_cell_formula_link       (Cell *cell);
 void        sheet_cell_formula_unlink     (Cell *cell);
+gboolean    sheet_is_region_empty         (Sheet *sheet, int start_col, int start_row,
+					   int end_col, int end_row);
 
 /* Create new ColRowInfos from the default sheet style */
 ColRowInfo *sheet_col_new                  (Sheet *sheet);
@@ -359,7 +363,7 @@ Workbook   *workbook_read                (const char *filename);
 void        workbook_save_as             (Workbook *);
 void        workbook_save                (Workbook *);
 void        workbook_attach_sheet        (Workbook *, Sheet *);
-void        workbook_detach_sheet        (Workbook *, Sheet *);
+gboolean    workbook_detach_sheet        (Workbook *, Sheet *);
 Sheet      *workbook_focus_current_sheet (Workbook *wb);
 Sheet      *workbook_get_current_sheet   (Workbook *wb);
 char       *workbook_sheet_get_free_name (Workbook *wb);
@@ -373,6 +377,7 @@ gboolean    workbook_rename_sheet        (Workbook *wb,
 					  const char *old_name,
 					  const char *new_name);
 int         workbook_sheet_count         (Workbook *wb);
+gboolean    workbook_can_detach_sheet    (Workbook *wb, Sheet *sheet);
 
 /*
  * Does any pending recalculations
