@@ -9,6 +9,7 @@
 #include <gnome.h>
 #include <gdk/gdkkeysyms.h>
 #include "gnumeric.h"
+#include "eval.h"
 #include "gnumeric-util.h"
 #include "gnumeric-sheet.h"
 #include "sheet-object.h"
@@ -215,6 +216,7 @@ workbook_do_destroy (Workbook *wb)
 
 	g_list_free (wb->eval_queue);
 
+	g_free (wb->toolbar);
 	if (wb->auto_expr){
 		expr_tree_unref (wb->auto_expr);
 		string_unref (wb->auto_expr_desc);
@@ -1134,7 +1136,7 @@ change_auto_expr_menu (GtkWidget *widget, GdkEventButton *event, Workbook *wb)
 					     _(quick_compute_routines [i].displayed_name));
 		}
 	}
-	gnumeric_popup_menu (GTK_MENU (menu), event);
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, 0, NULL, event->button, event->time);
 }
 
 /*
@@ -1275,7 +1277,10 @@ workbook_new (void)
 	gnome_app_set_contents (GNOME_APP (wb->toplevel), wb->table);
 	gnome_app_create_menus_with_data (GNOME_APP (wb->toplevel), workbook_menu, wb);
 	gnome_app_install_menu_hints(GNOME_APP (wb->toplevel), workbook_menu);
-	gnome_app_create_toolbar_with_data (GNOME_APP (wb->toplevel), workbook_toolbar, wb);
+
+	wb->toolbar = g_malloc (sizeof (workbook_toolbar));
+	memcpy (wb->toolbar, &workbook_toolbar, sizeof (workbook_toolbar));
+	gnome_app_create_toolbar_with_data (GNOME_APP (wb->toplevel), (GnomeUIInfo *) wb->toolbar, wb);
 
 	item = gnome_app_get_dock_item_by_name (GNOME_APP (wb->toplevel), GNOME_APP_TOOLBAR_NAME);
 	toolbar = gnome_dock_item_get_child (item);
@@ -1616,7 +1621,7 @@ workbook_attach_sheet (Workbook *wb, Sheet *sheet)
 }
 
 /**
- * workbook_detach_sheet:
+ * workbook_can_detach_sheet:
  * @wb: workbook.
  * @sheet: the sheet that we want to detach from the workbook
  *
@@ -1628,12 +1633,12 @@ workbook_can_detach_sheet (Workbook *wb, Sheet *sheet)
 {
 	GList *dependency_list;
 
-	g_return_if_fail (wb != NULL);
-	g_return_if_fail (sheet != NULL);
-	g_return_if_fail (IS_SHEET (sheet));
-	g_return_if_fail (sheet->workbook != NULL);
-	g_return_if_fail (sheet->workbook == wb);
-	g_return_if_fail (workbook_sheet_lookup (wb, sheet->name) == sheet);
+	g_return_val_if_fail (wb != NULL, FALSE);
+	g_return_val_if_fail (sheet != NULL, FALSE);
+	g_return_val_if_fail (IS_SHEET (sheet), FALSE);
+	g_return_val_if_fail (sheet->workbook != NULL, FALSE);
+	g_return_val_if_fail (sheet->workbook == wb, FALSE);
+	g_return_val_if_fail (workbook_sheet_lookup (wb, sheet->name) == sheet, FALSE);
 
 	dependency_list = region_get_dependencies (sheet, 0, 0, SHEET_MAX_COLS, SHEET_MAX_ROWS);
 	if (dependency_list == NULL)
@@ -1651,22 +1656,28 @@ workbook_can_detach_sheet (Workbook *wb, Sheet *sheet)
  *
  * Detaches @sheet from the workbook @wb.
  */
-void
+gboolean
 workbook_detach_sheet (Workbook *wb, Sheet *sheet)
 {
 	GtkNotebook *notebook;
 	int sheets, i;
 	
-	g_return_if_fail (wb != NULL);
-	g_return_if_fail (sheet != NULL);
-	g_return_if_fail (IS_SHEET (sheet));
-	g_return_if_fail (sheet->workbook != NULL);
-	g_return_if_fail (sheet->workbook == wb);
-	g_return_if_fail (workbook_sheet_lookup (wb, sheet->name) == sheet);
+	g_return_val_if_fail (wb != NULL, FALSE);
+	g_return_val_if_fail (sheet != NULL, FALSE);
+	g_return_val_if_fail (IS_SHEET (sheet), FALSE);
+	g_return_val_if_fail (sheet->workbook != NULL, FALSE);
+	g_return_val_if_fail (sheet->workbook == wb, FALSE);
+	g_return_val_if_fail (workbook_sheet_lookup (wb, sheet->name) == sheet, FALSE);
 
 	notebook = GTK_NOTEBOOK (wb->notebook);
 	sheets = workbook_sheet_count (sheet->workbook);
-	
+
+	if (sheets == 1)
+		return FALSE;
+
+	if (!workbook_can_detach_sheet (wb, sheet))
+		return FALSE;
+
 	/*
 	 * Remove our reference to this sheet
 	 */
@@ -1695,6 +1706,8 @@ workbook_detach_sheet (Workbook *wb, Sheet *sheet)
 	 * Queue a recalc
 	 */
 	workbook_recalc_all (wb);
+
+	return TRUE;
 }
 
 /**
@@ -1790,6 +1803,7 @@ void
 workbook_feedback_set (Workbook *workbook, WorkbookFeedbackType type, void *data)
 {
 	GtkToggleButton *t;
+	GnomeUIInfo *toolbar = workbook->toolbar;
 	int set;
 	
 	g_return_if_fail (workbook != NULL);
@@ -1797,7 +1811,7 @@ workbook_feedback_set (Workbook *workbook, WorkbookFeedbackType type, void *data
 	switch (type){
 	case WORKBOOK_FEEDBACK_BOLD:
 		t = GTK_TOGGLE_BUTTON (
-			workbook_toolbar [TOOLBAR_BOLD_BUTTON_INDEX].widget);
+			toolbar [TOOLBAR_BOLD_BUTTON_INDEX].widget);
 		set = data != NULL;
 
 		gtk_toggle_button_set_active (t, set);
@@ -1805,7 +1819,7 @@ workbook_feedback_set (Workbook *workbook, WorkbookFeedbackType type, void *data
 		
 	case WORKBOOK_FEEDBACK_ITALIC:
 		t = GTK_TOGGLE_BUTTON (
-			workbook_toolbar [TOOLBAR_ITALIC_BUTTON_INDEX].widget);
+			toolbar [TOOLBAR_ITALIC_BUTTON_INDEX].widget);
 		set = data != NULL;
 
 		gtk_toggle_button_set_active (t, set);
